/* where-hit.c -- given a position in one sequence of an alignment,
 *                find all matching positions in the other sequence
 *
 * This program examines an lav or concise alignment file to find all
 * matches for a specified position.  By default the given position
 * is with respect to the alignment's first sequence, and the program
 * reports the corresponding position(s) in the second sequence;
 * however this can be inverted using the "-inverse" option.  The
 * program also indicates the orientation of the match, and whether
 * it fell in a gap between segments of a local alignment.
 *
 * This program was suggested by Bob Lane.
 * 
 * $Revision: 1.10 $$Date: 2001/03/30 22:34:46 $
 */

#include "lav.h"
#include "util.h"

#define SYNTAX  "position align_file [-inverse] [-contig \">contig header\"] " \
                  "> output_file"

static void get_args (int argc, char **argv, int *pos, char **alignfile,
                      bool *inverse, char **contig);

int main (int argc, char **argv) {
	int pos = 0;
	char *alignfile = NULL;
	bool inverse = 0;
	char *contig = NULL;
	Segment *seg;
	int size;
	int i, pos2;
	bool gap;
	char *hdr2, *printed = NULL;
	int nhits = 0;

	/* get arguments */
	set_argv0 (argv[0]);
	get_args (argc, argv, &pos, &alignfile, &inverse, &contig);
	if (!pos || !alignfile)
		fatal (usage (SYNTAX));

	/* read alignment file into Segment array */
	size = get_align (alignfile, &seg);

	/* print title */
	printf ("%s found in %s for position %d",
	        (inverse ? "Inverse matches" : "Matches"), alignfile, pos);
	if (contig)
		printf (",\nin contig \"%s\"", contig);
	printf (":\n\n");

	/* find matches */
	i = -1;
	while ((i = find_seg (seg, pos, inverse, contig, i+1, &gap)) >= 0) {
		pos2 = convert_pos (seg, i, pos, inverse, gap);
		if (!(hdr2 = get_hdr2 (seg, i)))
			fatalf ("Missing sequence header for segment %d.", i);
		if (!strsame (hdr2, printed)) {
			printf ("%s\n", hdr2);
			printed = hdr2;
		}
		printf ("%10d", pos2);
		if (gap)
			printf (" (rounded due to gap)");
		if (seg[i].reverse)
			printf (" (aligned in reverse complement)");
		putchar ('\n');
		nhits++;
	}
	printf ("\n%d hit%s.\n", nhits, (nhits == 1 ? "" : "s"));

	/* clean up */
	free_align (seg, size);
	return 0;
}

/* Retrieve command-line arguments */
static void get_args (int argc, char **argv, int *pos, char **alignfile,
                      bool *inverse, char **contig) {
	int arg, i;

	for (i = 1; i < argc; i++) {
		if (strsame (argv[i], "-inverse"))
			*inverse = 1;
		else if (strsame (argv[i], "-contig"))
			*contig = optval (argc, argv, &i, SYNTAX);
		else if (!*pos && (arg = atoi (argv[i])) > 0)
			*pos = arg;
		else if (!*alignfile)
			*alignfile = argv[i];
		else
			fatal (usage (SYNTAX));
	}
}

